<?php
/**
 * @file
 * API class for Openmeetings.
 */

/*
 * You can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (
 * at your option) any later version.
 *
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can get a copy of the GNU General Public License
 * at <http://www.gnu.org/licenses/>.
 */

/**
 * License.
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License") +  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

require_once 'openmeetings_rest_service.php';
/**
 * Openmeetings Gateway API.
 */
class Openmeetingsgateway {
  public $sessionId = "";
  public $config;

  /**
   * Initialize config.
   */
  public function __construct($cfg) {
    $this->config = $cfg;
  }

  /**
   * Get the REST URL.
   */
  public function getRestUrl($name) {
    return $this->getUrl() . "/services/" . $name . "/";
  }

  /**
   * GET SOAP URL for ANonymous.
   *
   * Get SOAP login URL for the anonymous user. This URL is used for login
   * hash generated using gateway method getInvitationHash.
   *
   * @param string $invitation_hash
   *   The hash string generated.
   *
   * @return string
   *   Invitation URL
   */
  public function getInvitationUrl($invitation_hash) {
    return $this->getUrl() . "/swf?invitationHash=" . $invitation_hash;
  }

  /**
   * SOAP for Authneticated Users.
   *
   * Get SOAP login URL for the authenticated user. This URL is used for login
   * hash generated using gateway methods like setUserObjectAndGenerateRoomHash.
   *
   * @param string $invitation_hash
   *   The hash string generated.
   *
   * @return string
   *   Invitation URL
   */
  public function getSoapLoginUrl($invitation_hash) {
    return $this->getUrl() . "?secureHash=" . $invitation_hash;
  }

  /**
   * Get the Openmeetings base URL from config.
   */
  public function getUrl() {
    // FIXME protocol should be added.
    $port = $this->config["port"] == 80 ? '' : ":" . $this->config["port"];
    return $this->config["protocol"] . "://" . $this->config["host"] . $port . "/" . $this->config["webappname"];
  }
  /**
   * Check if boolean or not.
   */
  public function varToStr($in) {
    if (is_bool($in)) {
      return $in ? "true" : "false";
    }

    else {
      return $in;
    }
  }
  /**
   * Get session.
   */
  public function getSession() {
    $rest_service = new Openmeetingsrestservice();
    $response = $rest_service->call($this->getRestUrl("UserService") . "getSession", "session_id");
    $this->sessionID = $response;
  }
  /**
   * TODO: Get Error Service and show detailed Error Message.
   */
  public function loginuser() {
    $rest_service = new Openmeetingsrestservice();

    $this->getSession();

    if ($rest_service->getError()) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        $result = $rest_service->call($this->getRestUrl("UserService") . "loginUser?SID=" . $this->sessionID
            . "&username=" . urlencode($this->config["adminUser"])
            . "&userpass=" . urlencode($this->config["adminPass"]));
        if ($rest_service->getError()) {
          echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
          print_r($result);
          echo '</pre>';
        }
        else {
          $err = $rest_service->getError();
          if ($err) {
            echo '<h2>Error</h2><pre>' . $err . '</pre>';
          }
          else {
            $return_value = $result;
          }
        }
      }
    }

    if ($return_value > 0) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }
  /**
   * Update Room with moderation.
   */
  public function updateRoomWithModeration($openmeetings) {
    $rest_service = new Openmeetingsrestservice();
    $err = $rest_service->getError();
    if ($err) {
      echo '<h2>Constructor error</h2><pre>' . $err . '</pre>';
      echo '<h2>Debug</h2><pre>' . htmlspecialchars($client->getDebug(), ENT_QUOTES) . '</pre>';
      exit();
    }

    $is_moderated_room = FALSE;
    if ($openmeetings->is_moderated_room == 1) {
      $is_moderated_room = TRUE;
    }

    $result = $rest_service->call($this->getRestUrl("RoomService") . "updateRoomWithModeration?SID=" . $this->sessionID
        . "&room_id=" . $openmeetings->room_id . "&name=" . urlencode($openmeetings->roomname) . "&roomtypes_id="
        . urlencode($openmeetings->type) . "&comment=" . urlencode("Created by SOAP-Gateway")
        . "&numberOfPartizipants=" . $openmeetings->max_user . "&ispublic=false" . "&appointment=false" . "&isDemoRoom=false"
        . "&demoTime=0" . "&isModeratedRoom=" . $this->varToStr($is_moderated_room));

    if ($rest_service->fault) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        return $result;
      }
    }
    return -1;
  }
  /**
   * Update Room with moderation - questions, audiotype.
   */
  public function updateRoomWithModerationQuestionsAudioTypeAndHideOptions($openmeetings) {
    $rest_service = new Openmeetingsrestservice();

    $is_moderated_room = FALSE;
    if ($openmeetings->is_moderated_room == 1) {
      $is_moderated_room = TRUE;
    }

    $is_audio_only = "false";
    if ($openmeetings->is_audio_only == 1) {
      $is_audio_only = "true";
    }

    $allow_user_question = "false";
    if ($openmeetings->allow_user_question == 1) {
      $allow_user_question = "true";
    }

    $url = $this->getRestUrl("RoomService") . "updateRoomWithModerationQuestionsAudioTypeAndHideOptions?SID=" . $this->sessionID
      . "&room_id=" . $openmeetings->room_id . "&name=" . urlencode($openmeetings->roomname) . "&roomtypes_id="
      . urlencode($openmeetings->type) . "&comment=" . urlencode("Created by SOAP-Gateway")
      . "&numberOfPartizipants=" . $openmeetings->max_user . "&ispublic=false" . "&appointment=false"
      . "&isDemoRoom=false&hideTopBar=false&hideChat=false&hideActivitiesAndActions=false&hideFilesExplorer=false"
      . "&hideActionsMenu=false&hideScreenSharing=false&hideWhiteboard=false"
      . "&demoTime=0" . "&isModeratedRoom=" . $this->varToStr($is_moderated_room
      . '&allowUserQuestions=' . $allow_user_question . '&isAudioOnly=' . $is_audio_only);

    $result = $rest_service->call($url);

    if ($result) {
      return $result;
    }
    else {
      return -1;
    }
  }

  /**
   * Generate Recording Hash By URL.
   */
  public function setUserobjectandGeneraterecordingHashbyurl($username, $firstname, $lastname, $user_id, $system_type, $recording_id) {
    $rest_service = new Openmeetingsrestservice();
    $result = $rest_service->call($this->getRestUrl("UserService") . 'setUserobjectandGeneraterecordingHashbyurl?SID=' . $this->sessionID
        . '&username=' . urlencode($username) . '&firstname=' . urlencode($firstname) . '&lastname=' . urlencode($lastname)
        . '&externalUserId=' . $user_id . '&externalUserType=' . urlencode($system_type) . '&recording_id=' . $recording_id, 'return');

    if ($client_room_service->fault) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        return $result;
      }
    }
    return -1;
  }
  /**
   * Generate Recording Hash By URL And REC flag.
   */
  public function setUserObjectAndGenerateRoomHashByUrlAndRecFlag($username, $firstname, $lastname, $profile_picture_url, $email, $user_id, $system_type, $room_id, $become_moderator, $allow_recording) {
    $rest_service = new Openmeetingsrestservice();
    $err = $rest_service->getError();
    if ($err) {
      echo '<h2>Constructor error</h2><pre>' . $err . '</pre>';
      echo '<h2>Debug</h2><pre>' . htmlspecialchars($client->getDebug(), ENT_QUOTES) . '</pre>';
      exit();
    }

    $url = $this->getRestUrl("UserService") . "setUserObjectAndGenerateRoomHashByURLAndRecFlag?SID=" . $this->sessionID
        . "&username=" . urlencode($username) . "&firstname=" . urlencode($firstname) . "&lastname=" . urlencode($lastname)
        . "&profilePictureUrl=" . urlencode($profile_picture_url) . "&email=" . urlencode($email) . "&externalUserId=" . urlencode($user_id)
        . "&externalUserType=" . urlencode($system_type) . "&room_id=" . urlencode($room_id) . "&becomeModeratorAsInt=" . $become_moderator
        . "&showAudioVideoTestAsInt=1" . "&allowRecording=" . $this->varToStr($allow_recording);
    $result = $rest_service->call($url);
    return $result;
  }
  /**
   * Delete conference room.
   */
  public function deleteRoom($openmeetings) {
    $rest_service = new Openmeetingsrestservice();
    $err = $rest_service->getError();
    if ($err) {
      echo '<h2>Constructor error</h2><pre>' . $err . '</pre>';
      echo '<h2>Debug</h2><pre>' . htmlspecialchars($client->getDebug(), ENT_QUOTES) . '</pre>';
      exit();
    }

    $result = $rest_service->call($this->getRestUrl("RoomService") . "deleteRoom?SID=" . $this->sessionID
        . "&rooms_id=" . $openmeetings->room_id);

    if ($rest_service->fault) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        return $result;
      }
    }
    return -1;
  }

  /**
   * Generate a new room hash for entering a conference room.
   */
  public function setUserObjectAndGenerateRoomHash($username, $firstname, $lastname, $profile_picture_url, $email, $external_user_id, $external_user_type, $room_id, $become_moderator_as_int, $show_audio_video_test_as_int) {
    $rest_service = new Openmeetingsrestservice();

    $result = $rest_service->call($this->getRestUrl("UserService") . "setUserObjectAndGenerateRoomHash?SID=" . $this->sessionID
        . "&username=" . urlencode($username) . "&firstname=" . urlencode($firstname) . "&lastname=" . urlencode($lastname)
        . "&profilePictureUrl=" . urlencode($profile_picture_url) . "&email=" . urlencode($email) . "&externalUserId="
        . urlencode($external_user_id) . "&externalUserType=" . urlencode($external_user_type) . "&room_id=" . $room_id
        . "&becomeModeratorAsInt=" . $become_moderator_as_int . "&showAudioVideoTestAsInt=" . $show_audio_video_test_as_int);

    $err = $rest_service->getError();
    if ($err) {
      echo '<h2>Constructor error</h2><pre>' . $err . '</pre>';
      echo '<h2>Debug</h2><pre>' . htmlspecialchars($client->getDebug(), ENT_QUOTES) . '</pre>';
      exit();
    }

    if ($rest_service->getError()) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        return $result;
      }
    }
    return -1;
  }

  /**
   * Create a new conference room.
   */
  public function createRoomWithModAndType($openmeetings) {
    global $_openmeetings_user;

    $rest_service = new Openmeetingsrestservice();

    $is_moderated_room = "false";
    if ($openmeetings->is_moderated_room == 1) {
      $is_moderated_room = "true";
    }

    $url = $this->getRestUrl("RoomService") . 'addRoomWithModerationAndExternalType?SID=' . $this->sessionID
        . '&name=' . urlencode($openmeetings->roomname) . '&roomtypes_id=' . $openmeetings->type . '&comment='
        . urlencode('Created by SOAP-Gateway') . '&numberOfPartizipants=' . $openmeetings->max_user
        . '&ispublic=false' . '&appointment=false' . '&isDemoRoom=false' . '&demoTime=0' . '&isModeratedRoom=' . $is_moderated_room
        . '&externalRoomType=' . urlencode($this->config["moduleKey"]);

    $result = $rest_service->call($url, "return");

    if ($rest_service->fault) {
      echo '<h2>Fault (Expect - The request contains an invalid SOAP body)</h2><pre>';
      print_r($result);
      echo '</pre>';
    }
    else {
      $err = $rest_service->getError();
      if ($err) {
        echo '<h2>Error</h2><pre>' . $err . '</pre>';
      }
      else {
        return $result;
      }
    }
    return -1;
  }

  /**
   * Create a new conference room with Moderation, External type and audio type.
   *
   * @param object $openmeetings
   *   Openmeetings object with following values
   *   roomname -> Name of Room, String
   *   type -> Room type ID, integer (1 = Conference, 2 = Audience,
   *             3 = Restricted, 4 = Interview)
   *   max_user            -> Maximum number of user, integer.
   *   is_moderated_room   -> Is room moderated, boolean
   *   is_audio_only       -> Is room audio only, boolean
   *   allow_user_question -> Allow user question, boolean.
   */
  public function addRoomWithModerationExternalTypeAndAudioType($openmeetings) {
    $rest_service = new Openmeetingsrestservice();

    $is_moderated_room = "false";
    if ($openmeetings->is_moderated_room == 1) {
      $is_moderated_room = "true";
    }

    $is_audio_only = "false";
    if ($openmeetings->is_audio_only == 1) {
      $is_audio_only = "true";
    }

    $allow_user_question = "false";
    if ($openmeetings->allow_user_question == 1) {
      $allow_user_question = "true";
    }

    $url = $this->getRestUrl("RoomService") . 'addRoomWithModerationExternalTypeAndAudioType?SID=' . $this->sessionID
        . '&name=' . urlencode($openmeetings->roomname) . '&roomtypes_id=' . $openmeetings->type . '&comment='
        . urlencode('Created by SOAP-Gateway') . '&numberOfPartizipants=' . $openmeetings->max_user
        . '&ispublic=false' . '&appointment=false' . '&isDemoRoom=false' . '&demoTime=0' . '&isModeratedRoom=' . $is_moderated_room
        . '&allowUserQuestions=' . $allow_user_question . '&isAudioOnly=' . $is_audio_only
        . '&externalRoomType=' . urlencode($this->config["moduleKey"]);

    $result = $rest_service->call($url, "return");

    if ($result) {
      return $result;
    }
    else {
      return -1;
    }
  }

  /**
   * Get invitation hash for a room.
   *
   * @param object $openmeetings
   *   Openmeetings Object.
   *
   * @return string
   *   Invitation hash.
   */
  public function getInvitationHashFullName($openmeetings) {
    $rest_service = new Openmeetingsrestservice();
    $is_password_protected = 'false';
    if ($openmeetings->isPasswordProtected) {
      $is_password_protected = 'true';
    }
    $valid_from_date = $valid_from_time = $valid_to_date = $valid_to_time = '';
    if ($openmeetings->valid == 2) {
      if ($openmeetings->from_date) {
        $valid_from_date = $openmeetings->from_date;
        $valid_from_time = $openmeetings->from_time;
      }
      if ($openmeetings->to_date) {
        $valid_to_date = $openmeetings->to_date;
        $valid_to_time = $openmeetings->to_time;
      }
    }

    $url = $this->getRestUrl("RoomService") . 'getInvitationHashFullName?SID=' . $this->sessionID
      . '&username=' . urlencode($openmeetings->username) . '&firstname=' . urlencode($openmeetings->firstname)
      . '&lastname=' . urlencode($openmeetings->lastname) . '&room_id=' . urlencode($openmeetings->room_id)
      . '&isPasswordProtected=' . $is_password_protected . '&invitationpass=' . urlencode($openmeetings->invitationpass)
      . '&valid=' . $openmeetings->valid . '&validFromDate=' . urlencode($valid_from_date) . '&validFromTime=' . urlencode($valid_from_time)
      . '&validToDate=' . urlencode($valid_to_date) . '&validToTime=' . urlencode($valid_to_time);

    $result = $rest_service->call($url, "return");

    if ($result) {
      return $result;
    }
    else {
      return -1;
    }
  }
  /**
   * Add a existing to usergroup.
   *
   * @param int $room_id
   *   Room ID to be added to usergroup.
   * @param int $org_id
   *   User group ID to which room is to be added.
   */
  public function addRoomToOrg($room_id, $org_id) {
    $rest_service = new Openmeetingsrestservice();
    $url = $this->getRestUrl("RoomService") . 'addRoomToOrg?SID=' . $this->sessionID
      . '&rooms_id=' . $room_id . '&organisation_id=' . $org_id;
    $result = $rest_service->call($url, "return");

    if ($result) {
      return $result;
    }
    else {
      return -1;
    }
  }
  /**
   * Return an array.
   */
  public function getList($result) {
    return $result && !is_array(reset($result)) ? array($result) : $result;
  }

  /**
   * Get list of available recordings made by this instance.
   */
  public function getRecordingsByExternalRooms() {
    $rest_service = new Openmeetingsrestservice();

    $url = $this->getRestUrl("RoomService") . "getFlvRecordingByExternalRoomType?SID=" . $this->sessionID
      . "&externalRoomType=" . urlencode($this->config["moduleKey"]);

    $result = $rest_service->call($url, "return");

    return $this->getList($result);
  }

  /**
   * Get list of available recordings made by user.
   */
  public function getRecordingsByExternalUser($id) {
    $rest_service = new Openmeetingsrestservice();

    $url = $this->getRestUrl("RoomService") . "getFlvRecordingByExternalUserId?SID=" . $this->sessionID
      . "&externalUserId=" . $id;

    $result = $rest_service->call($url, "return");

    return $this->getList($result);
  }

  /**
   * Function to add a new user.
   */
  public function addNewUser($openmeeting) {
    $rest_service = new Openmeetingsrestservice();
    $url = $this->getRestUrl("UserService") . "addNewUser?SID=" . $this->sessionID
        . "&username=" . urlencode($openmeeting['username']) . "&userpass=" . urlencode($openmeeting['userpass']) . "&firstname=" . urlencode($openmeeting['firstname'])
        . "&lastname=" . urlencode($openmeeting['lastname']) . "&email=" . urlencode($openmeeting['email']) . "&additionalname=" . urlencode($openmeeting['additionalname'])
        . "&street=" . urlencode($openmeeting['street']) . "&zip=" . urlencode($openmeeting['zip']) . "&fax=" . urlencode($openmeeting['fax'])
        . "&states_id=98" . "&town=" . urlencode($openmeeting['town']) . "&language_id=1";
    $result = $rest_service->call($url);
    return $result;
  }

  /**
   * Get error by error code.
   *
   * @param int $errorid
   *
   *   It will be a negative value.
   *
   * @return array
   *   contains error information
   */
  public function getErrorByCode($errorid) {
    $rest_service = new Openmeetingsrestservice();

    $url = $this->getRestUrl("UserService") . "getErrorByCode?SID=" . $this->sessionID
      . "&errorid=" . urlencode($errorid) . "&language_id=1";
    $result = $rest_service->call($url);
    return $result;
  }

}
