<?php

/**
 * @file
 * Rest service file of openmeetings module.
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.  The ASF licenses this
 * file to you under the Apache License, Version 2.0 (the
 * "License") +  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/**
 * Created on 03.01.2012 by eugen.schwert@gmail.com.
 */
class Openmeetingsrestservice {
  /**
   * REST call.
   *
   * @param $request
   *   Initialize the session by passing the request as a parameter.
   */
  public function call($request, $return_attribute = "return") {
    // This will allow you to view errors in the browser
    // Note: set "display_errors" to 0 in production
    // ini_set('display_errors',1);
    // Report all PHP errors (notices, errors, warnings, etc.)
    // error_reporting(E_ALL);
    // URI used for making REST call. Each Web Service uses a unique URL.
    $options = array(
      // Return web page.
      CURLOPT_RETURNTRANSFER => TRUE,
      // Return headers.
      CURLOPT_HEADER => TRUE,
      // Follow redirects.
      CURLOPT_FOLLOWLOCATION => TRUE,
      // Handle all encodings.
      CURLOPT_ENCODING => "",
      // Who am i.
      CURLOPT_USERAGENT => "openmeetings",
      // Set referer on redirect.
      CURLOPT_AUTOREFERER => TRUE,
      // Timeout on connect.
      CURLOPT_CONNECTTIMEOUT => 120,
      // Timeout on response.
      CURLOPT_TIMEOUT => 120,
      // Stop after 10 redirects.
      CURLOPT_MAXREDIRS => 10,
    );
    $session = curl_init($request);
    curl_setopt_array($session, $options);

    // Make the request.
    $response = curl_exec($session);

    // Confirm that the request was transmitted to the OpenMeetings!
    // Image Search Service.
    if (!$response) {
      $err = curl_errno($session);
      $errmsg = curl_error($session);
      $header = curl_getinfo($session);
      throw new Exception(curl_error($session));
    }
    // Close the curl session.
    curl_close($session);

    // Create an array to store the HTTP response codes.
    $status_code = array();

    // Use regular expressions to extract the code from the header.
    preg_match('/\d\d\d/', $response, $status_code);

    // Check the HTTP Response code and display message
    // if status code is not 200 (OK).
    switch ($status_code[0]) {
      // Success.
      case 200:
        break;

      case 503:
        throw new Exception('Your call to OpenMeetings Web Services failed and returned an HTTP status of 503. That means: Service unavailable. An internal problem prevented us from returning data to you.');

      break;
      case 403:
        throw new Exception('Your call to OpenMeetings Web Services failed and returned an HTTP status of 403. That means: Forbidden. You do not have permission to access this resource, or are over your rate limit.');

      break;
      case 400:
        // You may want to fall through here and read the specific XML error.
        throw new Exception('Your call to OpenMeetings Web Services failed and returned an HTTP status of 400. That means:  Bad request. The parameters passed to the service did not match as expected. The exact error is returned in the XML response.');

      break;
      default:
        throw new Exception('Your call to OpenMeetings Web Services returned an unexpected HTTP status of: ' . $status_code[0] . " Request " . $request);
    }

    // Get the XML from the response, bypassing the header.
    if (!($xml = strstr($response, '<ns'))) {
      $xml = NULL;
    }

    $dom = new DOMDocument();
    $dom->loadXML($xml);

    if ($return_attribute == "") {
      return $this->getarray($dom);
    }
    else {
      $return_node_list = $dom->getElementsByTagName($return_attribute);
      $ret = array();
      foreach ($return_node_list as $return_node) {
        if ($return_node_list->length == 1) {
          return $this->getarray($return_node);
        }
        else {
          $ret[] = $this->getarray($return_node);
        }
      }
      return $ret;
    }
  }
  /**
   * Get array of node.
   */
  public function getarray($node) {
    if (is_null($node) || !is_object($node)) {
      return $node;
    }
    $array = FALSE;
    if ($node->hasChildNodes()) {
      foreach ($node->childNodes as $child_node) {
        if ($child_node->nodeType != XML_TEXT_NODE) {
          if ($node->hasAttributes()) {
            foreach ($node->attributes as $attr) {
              if ($attr->localName == "nil") {
                return NULL;
              }
            }
          }
          if ($child_node->childNodes->length == 1) {
            $array[$child_node->localName] = $this->getarray($child_node);
          }
          else {
            $array[$child_node->localName][] = $this->getarray($child_node);
          }
        }
        else {
          return $child_node->nodeValue;
        }
      }
    }

    return $array;
  }

  /**
   * Get the error.
   */
  public function geterror() {
    return FALSE;
  }

  /**
   * Get the fault.
   */
  public function fault() {
    return FALSE;
  }

}
