<?php

/**
 * @file
 * Openmeetings Admin pages.
 */

// Includes ctools-object-cache.
ctools_include('object-cache');

/**
 * Page callback for Openmeetings settings form.
 */
function openmeetings_settings() {
  $form = array();
  $form['openmeetings_user_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting Username'),
    '#default_value' => variable_get('openmeetings_user_name', ''),
  );
  $form['openmeetings_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting Password'),
    '#default_value' => variable_get('openmeetings_password', ''),
  );
  $form['openmeetings_protocol'] = array(
    '#type' => 'radios',
    '#title' => t('HTTP Protocol'),
    '#options' => array('http' => t('http'), 'https' => t('https')),
    '#default_value' => variable_get('openmeetings_protocol', ''),
  );
  $form['openmeetings_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting Host'),
    '#default_value' => variable_get('openmeetings_host', ''),
  );
  $form['openmeetings_webapp_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting Web Application Name'),
    '#default_value' => variable_get('openmeetings_webapp_name', ''),
  );
  $form['openmeetings_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting PORT'),
    '#default_value' => variable_get('openmeetings_port', ''),
  );
  $form['openmeetings_user_group_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Openmeeting Usergroup ID'),
    '#default_value' => variable_get('openmeetings_user_group_id', ''),
  );
  $form['openmeetings_timezone'] = array(
    '#type' => 'select',
    '#title' => t('Openmeeting Timezone'),
    '#default_value' => variable_get('openmeetings_timezone', ''),
    '#options' => system_time_zones(),
  );

  $form['openmeetings_conference_invitation_mail_format'] = array(
    '#type' => 'fieldset',
    '#title' => t('Conference Invitation mail format'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE,
  );
  $invitation_mail_details = variable_get('openmeetings_conference_invitation_mail_format', array());
  $form['openmeetings_conference_invitation_mail_format']['openmeetings_conference_email_from'] = array(
    '#type' => 'textfield',
    '#title' => t('From'),
    '#default_value' => isset($invitation_mail_details['openmeetings_conference_email_from']) ? $invitation_mail_details['openmeetings_conference_email_from'] : '',
  );
  $form['openmeetings_conference_invitation_mail_format']['openmeetings_conference_email_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => isset($invitation_mail_details['openmeetings_conference_email_subject']) ? $invitation_mail_details['openmeetings_conference_email_subject'] : '',
  );
  $form['openmeetings_conference_invitation_mail_format']['openmeetings_conference_email_cc'] = array(
    '#type' => 'textfield',
    '#title' => t('CC'),
    '#default_value' => isset($invitation_mail_details['openmeetings_conference_email_cc']) ? $invitation_mail_details['openmeetings_conference_email_cc'] : '',
  );
  $form['openmeetings_conference_invitation_mail_format']['openmeetings_conference_email_bcc'] = array(
    '#type' => 'textfield',
    '#title' => t('BCC'),
    '#default_value' => isset($invitation_mail_details['openmeetings_conference_email_bcc']) ? $invitation_mail_details['openmeetings_conference_email_bcc'] : '',
  );
  $form['openmeetings_conference_invitation_mail_format']['openmeetings_conference_email_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => isset($invitation_mail_details['openmeetings_conference_email_body']) ? $invitation_mail_details['openmeetings_conference_email_body'] : '',
  );
  $postponed_meeting_details = variable_get('openmeetings_conference_sending_postponed_meeting_details', array());
  $form['openmeetings_conference_sending_postponed_meeting_details'] = array(
    '#type' => 'fieldset',
    '#title' => t('Conference Mail format which is postponed/Cancelled.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE,
  );
  $form['openmeetings_conference_sending_postponed_meeting_details']['openmeetings_conference_email_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => isset($postponed_meeting_details['openmeetings_conference_email_subject']) ? $postponed_meeting_details['openmeetings_conference_email_subject'] : '',
  );
  $form['openmeetings_conference_sending_postponed_meeting_details']['openmeetings_conference_email_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => isset($postponed_meeting_details['openmeetings_conference_email_body']) ? $postponed_meeting_details['openmeetings_conference_email_body'] : '',
  );
  $markup = '
  <p>You can use the following tokens in the email body.</p>
  <ul>
    <li>[openmeetings:conference-link] = Conference Link.</li>
    <li>[openmeetings:conference-title] = Title of conference</li>
    <li>[openmeetings:conference-description] = Description of conference</li>
    <li>[openmeetings:conference-start-time] = Conference start time</li>
    <li>[openmeetings:conference-end-time] = Conference end time</li>
  </ul>
  ';
  $form['openmeetings_conference_invitation_mail_format']['useful_tokens'] = array(
    '#type' => 'item',
    '#title' => t('Useful Tokens'),
    '#markup' => $markup,
  );
  $form['openmeetings_conference_sending_postponed_meeting_details']['useful_tokens'] = array(
    '#type' => 'item',
    '#title' => t('Useful Tokens'),
    '#markup' => $markup,
  );
  return system_settings_form($form);
}

/**
 * Page callback to create a new conference.
 */
function openmeetings_conference_rooms_new_conference() {
  return drupal_get_form('openmeetings_conference_rooms_new_conference_form');
}

/**
 * Function callback for adding a new conference.
 */
function openmeetings_conference_rooms_new_conference_form($form, &$form_state, $conference_id = NULL) {
  $terms = array();
  if ($conference_id) {
    $conference = openmeetings_get_conference_details($conference_id);
  }
  $form = array();

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Conference Title'),
    '#required' => TRUE,
    '#description' => t('Add a unique conference title'),
    '#default_value' => isset($conference->title) ? $conference->title : "",
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('Add a small description about the conference'),
    '#default_value' => isset($conference->description) ? $conference->description : "",
  );

  if (isset($conference->start_time)) {
    $date = format_date($conference->start_time, 'custom', 'Y-m-d H:i:s');
  }
  else {
    $date = format_date(REQUEST_TIME, 'custom', 'Y-m-d H:i:s');
  }
  $timezone = variable_get('openmeetings_timezone', '');
  // Provide a format using regular PHP format parts.
  // If you're using a date_select,
  // the format will control the order of the date parts in the selector,
  // rearrange them any way you like.
  // Parts left out of the format will not be displayed to the user.
  $format = 'Y-m-d H:i';

  $form['start_time'] = array(
    '#type' => 'date_popup',
    '#title' => t('Select start time of meeting'),
    '#description' => t("Meeting start time is scheduled on openmeetings' Time zone, which is @timezone", array('@timezone' => $timezone)),
    '#default_value' => $date,
    '#date_format' => $format,
    '#date_label_position' => 'within',
    '#date_year_range' => '-3:+3',
    '#datepicker_options' => array(
      'minDate' => 0,
      'changeMonth' => FALSE,
      'changeYear' => FALSE,
    ),
  );

  if (isset($conference->end_time)) {
    $date = format_date($conference->end_time, 'custom', 'Y-m-d H:i:s');
  }
  else {
    $date = format_date(REQUEST_TIME, 'custom', 'Y-m-d H:i:s');
  }

  $form['end_time'] = array(
    '#type' => 'date_popup',
    '#title' => t('Select end time of meeting'),
    '#description' => t("Meeting end time is scheduled on openmeetings' Time zone, which is @timezone", array('@timezone' => $timezone)),
    '#default_value' => $date,
    '#date_format' => $format,
    '#ajax' => array(
      'callback' => 'openmeetings_conference_rooms_new_conference_form_validate_end_time',
      'event' => 'change',
    ),
    '#date_label_position' => 'within',
    '#datepicker_options' => array(
      'minDate' => 0,
      'changeMonth' => FALSE,
      'changeYear' => FALSE,
    ),
    '#date_label_position' => 'within',
    '#date_year_range' => '-3:+3',
  );
  if ($conference_id) {
    $form['notify_user'] = array(
      '#type' => 'checkbox',
      '#title' => t('Notify Already Invited User'),
      '#default_value' => isset($conference->notify_user) ? $conference->notify_user : '',
    );
  }

  $conference_types = get_conference_types();
  $form['type'] = array(
    '#type' => 'select',
    '#title' => t('Conference Type'),
    '#options' => $conference_types,
    '#required' => TRUE,
    '#description' => t('Select the conference type, If Interview - 1:1 Meeting is selected,
                       then maximum number of participants should be 2, You can either select 2 non members
                       (Is conference Moderated checkbox should be unchecked) OR
                       one non-member and a authenticated user as moderator'),
    '#default_value' => isset($conference->type) ? $conference->type : "",
  );
  if (isset($conference->is_moderated) && $conference->is_moderated == 1) {
    $conference->conference_member_limit = $conference->conference_member_limit - 2;
  }

  $form['conference_member_limit'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of Participants'),
    '#required' => TRUE,
    '#default' => t('0'),
    '#description' => t('Enter maximum number of participants (Includes only participants not moderators)'),
    '#default_value' => isset($conference->conference_member_limit) ? $conference->conference_member_limit : "",
  );

  $form['is_moderated'] = array(
    '#type' => 'checkbox',
    '#title' => t('Is conference Moderated'),
    '#default_value' => isset($conference->is_moderated) ? $conference->is_moderated : 1,
    '#ajax' => array(
      'callback' => 'openmeetings_conference_rooms_new_conference_form_hide_moderator',
      'effect' => 'fade',
    ),
  );

  $form['is_audio_only'] = array(
    '#type' => 'checkbox',
    '#title' => t('Is conference audio only'),
    '#default_value' => isset($conference->is_audio_only) ? $conference->is_audio_only : "",
  );

  $form['allow_user_questions'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow user questions'),
    '#default_value' => isset($conference->allow_user_questions) ? $conference->allow_user_questions : "",
  );

  $form['allow_recording'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow recording'),
    '#default_value' => isset($conference->allow_recording) ? $conference->allow_recording : "",
  );

  $form['not_member'] = array(
    '#type' => 'fieldset',
    '#title' => t('Not a Member!'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#prefix' => '<div id="conference-emails">',
    '#suffix' => '</div>',
  );

  $form['not_member']['email'] = array(
    '#type' => 'textfield',
    '#title' => t('Enter email address'),
    '#default_value' => '',
  );

  $form['not_member']['add'] = array(
    '#type' => 'button',
    '#value' => t('Add Email'),
    '#ajax' => array(
      'callback' => 'openmeetings_conference_rooms_new_conference_form_ajax',
      'effect' => 'fade',
    ),
    '#limit_validation_errors' => array(array('email')),
    '#submit' => array(),
  );

  $form['not_member']['added_emails'] = array(
    '#type' => 'fieldset',
    '#title' => t('Added Email Addresses'),
    '#tree' => TRUE,
    '#collapsible' => FALSE,
    '#prefix' => '<div id="conference-email-list">',
    '#suffix' => '</div>',
  );

  // Add new emails in the list using AJAX.
  $added_emails = array();

  // In case we are using the form for editing the conference.
  if (isset($conference->users) && !empty($conference->users)) {
    foreach ($conference->users as $conf_user_value) {
      if ($conf_user_value['uid'] == 0) {
        if ($conf_user_value['email'] != '') {
          $added_emails[$conf_user_value['email']] = $conf_user_value['email'];
        }
      }
    }
    $form['not_member']['added_emails']['emails']['currently_added_emails'] = array(
      '#type' => 'item',
      '#markup' => t('Currently added emails are: @emails', array('@emails' => implode(', ', $added_emails))),
    );
    $added_email_count = count($added_emails);
  }

  $added_emails = ctools_object_cache_get('openmeetings', 'openmeetings_added_emails');
  $user_count = ctools_object_cache_get('openmeetings', 'openmeetings_registered_participants');

  if (isset($form_state['input']['conference_member_limit']) && ($form_state['input']['conference_member_limit'] != '')) {

    if (!isset($conference->conference_member_limit)) {
      $limit = $form_state['input']['conference_member_limit'] - $user_count;
      $added_email_count = isset($form_state['input']['added_emails']['emails']) ? count($form_state['input']['added_emails']['emails']) : 0;
    }
    else {
      $limit = $conference->conference_member_limit - $user_count;
      if (isset($form_state['input']['added_emails']['emails']) && count($form_state['input']['added_emails']['emails']) != 0) {
        $added_email_count = $added_email_count + count($form_state['input']['added_emails']['emails']);
      }
    }

    $new_email = isset($form_state['values']['email']) ? $form_state['values']['email'] : '';
    if (filter_var($new_email, FILTER_VALIDATE_EMAIL) == '') {
      $form_state['validate_error'] = 1;
    }
    elseif ($added_email_count == $limit) {
      $form_state['values']['email'] = '';
      $form_state['validate_error'] = 2;
    }
    else {
      $form_state['validate_error'] = 0;
      $added_emails[$new_email] = $new_email;
      $form_state['added_emails'] = $added_emails;
      ctools_object_cache_set('openmeetings', 'openmeetings_added_emails', $added_emails);
    }
  }

  if (!empty($added_emails)) {
    foreach ($added_emails as $added_email) {
      $form['not_member']['added_emails']['emails'][$added_email] = array(
        '#type' => 'checkbox',
        '#title' => $added_email,
      );
    }
    $form['not_member']['added_emails']['remove_email'] = array(
      '#type' => 'button',
      '#value' => t('Remove Selected Emails'),
      '#ajax' => array(
        'callback' => 'openmeetings_conference_rooms_new_conference_form_remove_email_ajax',
        'wrapper' => 'conference-email-list',
        'method' => 'replace',
        'effect' => 'fade',
        'event' => 'click',
      ),
      '#limit_validation_errors' => array(),
    );

    if (isset($form['not_member']['added_emails']['emails'][0])) {
      $form['not_member']['added_emails']['emails'][0]['#markup'] = '';
    }
  }
  else {
    $form['not_member']['added_emails']['emails'][0] = array(
      '#markup' => t('No Emails added'),
    );
  }
  $moderator_name_value = '';
  if (isset($conference->moderators) && !empty($conference->moderators)) {
    $moderator_uid = $conference->moderators[0];
    $moderator_name = db_select('users', 'u')
            ->fields('u', array('name'))
            ->condition('uid', $moderator_uid)
            ->execute()->fetchField();
    $moderator_name_value = $moderator_name . ' (uid=' . $moderator_uid . ')';
  }

  $is_moderated = 1;
  if (count($form_state['input']) > 0) {
    if ($form_state['input']['is_moderated'] == '') {
      $is_moderated = 0;
    }
  }
  elseif ((isset($conference->is_moderated) != '') && ($conference->is_moderated == 0)) {
    $is_moderated = 0;
  }

  if ($is_moderated == 1) {
    // Adding moderators to conference.
    $form['moderator'] = array(
      '#type' => 'fieldset',
      '#title' => t('Moderator'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#prefix' => '<div id="conference-moderators">',
      '#suffix' => '</div>',
    );

    $form['moderator']['moderator_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#description' => t('Only the registered users in this site will become moderators'),
      '#size' => 30,
      '#autocomplete_path' => 'ajax/openmeetings/user_autocomplete_callback',
      '#default_value' => $moderator_name_value,
      '#weight' => -1,
      '#required' => TRUE,
    );
  }
  if ($conference_id) {
    $form['disable_conference'] = array(
      '#type' => 'checkbox',
      '#title' => t('Disable conference'),
      '#default_value' => 0,
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create Conference'),
  );

  // Retrieve the selected emails to be removed.
  if (isset($form_state['input']['added_emails'])) {
    $selected_emails_to_remove = array_keys(array_filter($form_state['input']['added_emails']['emails']));
    $added_emails = ctools_object_cache_get('openmeetings', 'openmeetings_added_emails');
    // Remove the emails from both cache and form.
    foreach ($selected_emails_to_remove as $email_to_remove) {
      unset($added_emails[$email_to_remove]);
      unset($form['not_member']['added_emails']['emails'][$email_to_remove]);
    }
    ctools_object_cache_set('openmeetings', 'openmeetings_added_emails', $added_emails);
  }

  if (isset($conference->cid)) {
    $form['submit']['#value'] = t('Save Changes');
    $form['conference_id'] = array(
      '#type' => 'hidden',
      '#value' => $conference->cid,
    );
    $form['remote_conference_id'] = array(
      '#type' => 'hidden',
      '#value' => $conference->om_cid,
    );
    $form['cancel'] = array(
      '#type' => 'submit',
      '#value' => t('Cancel'),
      '#submit' => array('openmeetings_conference_rooms_new_conference_form_cancel'),
      '#limit_validation_errors' => array(),
    );
  }
  $form['#validate'][] = 'openmeetings_conference_rooms_new_conference_form_validate';
  return $form;
}

/**
 * Ajax callback for adding a new email id.
 */
function openmeetings_conference_rooms_new_conference_form_ajax($form, &$form_state) {
  if ($form_state['validate_error'] == 1) {
    $commands[] = ajax_command_before('#conference-emails', '<div class="messages error" id="validate_error">Enter valid email address.</div>');
  }
  if ($form_state['validate_error'] == 2) {
    $commands[] = ajax_command_before('#conference-emails', '<div class="messages error" id="validate_error">Number of participant for this conference is limited to ' . $form_state['values']['conference_member_limit'] . '</div>');
  }
  if ($form_state['validate_error'] == 0) {
    $commands[] = ajax_command_remove('#validate_error');
  }
  $commands[] = ajax_command_replace('#conference-email-list', render($form['not_member']['added_emails']));
  $commands[] = ajax_command_invoke('#edit-email', "val", array(""));
  return array('#type' => 'ajax', '#commands' => $commands);
}

/**
 * Ajax callback to validate start time.
 */
function openmeetings_conference_rooms_new_conference_form_validate_start_time($form, &$form_state) {
  $start_date = $form_state['input']['start_time']['date'] . ' ' . $form_state['input']['start_time']['time'];
  $end_date = $form_state['input']['end_time']['date'] . ' ' . $form_state['input']['end_time']['time'];
  if ($start_date > $end_date) {
    $commands[] = ajax_command_replace('messages error', '');
    $commands[] = ajax_command_before('.form-item-start-time', '<div class="messages error">Start Time should be lower than the End Time.</div>');
  }
  else {
    $commands[] = ajax_command_remove('.container-inline-date .messages');
  }
  return array('#type' => 'ajax', '#commands' => $commands);
}

/**
 * Ajax callback to validate end time.
 */
function openmeetings_conference_rooms_new_conference_form_validate_end_time($form, &$form_state) {
  $selected_date = $form_state['input']['end_time']['date'] . ' ' . $form_state['input']['end_time']['time'];
  $start_date = $form_state['input']['start_time']['date'] . ' ' . $form_state['input']['start_time']['time'];
  if ($selected_date < $start_date) {
    $commands[] = ajax_command_replace('messages error', '');
    $commands[] = ajax_command_before('.form-item-end-time', '<div class="messages error">End Time should not be less than the Start Time.</div>');
  }
  else {
    $commands[] = ajax_command_remove('.container-inline-date .messages');
  }
  return array('#type' => 'ajax', '#commands' => $commands);
}

/**
 * AJAX callback for removing the selected emails.
 */
function openmeetings_conference_rooms_new_conference_form_remove_email_ajax($form, $form_state) {
  return $form['not_member']['added_emails'];
}

/**
 * Ajax callback for adding a new moderator.
 */
function openmeetings_conference_rooms_new_conference_add_moderator($form, &$form_state) {
  $commands[] = ajax_command_replace('#conference-email-list', render($form['not_member']['added_emails']));
  $commands[] = ajax_command_invoke('#edit-email', "val", array(""));
  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );
}

/**
 * Ajax callback for hiding moderator block.
 */
function openmeetings_conference_rooms_new_conference_form_hide_moderator($form, &$form_state) {
  if ($form_state['values']['is_moderated'] == 1) {
    $commands[] = ajax_command_css('#conference-moderators', array('display' => 'block'));
  }
  elseif ($form_state['values']['is_moderated'] == 0) {
    $commands[] = ajax_command_css('#conference-moderators', array('display' => 'none'));
  }
  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );
}

/**
 * Ajax callback for getting number of uses added to conference.
 */
function openmeetings_conference_rooms_new_conference_form_find_no_of_users_ajax($form, &$form_state) {
  if (isset($form_state['user_count'])) {
    if ($form_state['triggering_element']['#value'] != 0) {
      $commands[] = ajax_command_replace('#registered_participants', '');
      $commands[] = ajax_command_after('.form-item-conference-member-limit', '<div class="messages status" id="registered_participants">We have selected ' . $form_state['user_count'] . '
      registered users to attend this conference. If it is not equal to maximum number of participants,
      we can add non-members to attend the conference.</div>');
    }
    else {
      $commands[] = ajax_command_replace('#registered_participants', '<div class="messages status" id="registered_participants">We have selected ' . $form_state['user_count'] . '
      registered users to attend this conference. If it is not equal to maximum number of participants,
      we can add non-members to attend the conference.</div>');
    }
  }
  return array(
    '#type' => 'ajax',
    '#commands' => $commands,
  );
}


/**
 * Validate function for the form for creating the conference.
 */
function openmeetings_conference_rooms_new_conference_form_validate($form, &$form_state) {
  $editing = FALSE;
  if (isset($form_state['values']['conference_id'])) {
    $editing = TRUE;
    $cid = $form_state['values']['conference_id'];
  }
  // Get the value of email field,
  // if you are under than use: "dpm($form_state['values'])".
  $title = $form_state['values']['title'];

  // Find whether the title exist or not.
  $query = db_select('openmeetings_conference', 'c');
  $query->fields('c', array('cid'));
  $query->condition('c.title', $title);
  if ($editing) {
    $query->condition('c.cid', $cid, "!=");
  }

  // This will return the count of the fields found.
  $row_count = $query->countQuery()->execute()->fetchField();

  if (isset($row_count) && ($row_count > 0)) {
    form_set_error('title', t('This title already exists, please try with another title'));
  }

  $e_date = strtotime($form_state['complete form']['start_time']['#default_value']);
  $c_date = strtotime($form_state['complete form']['start_time']['#value']['date'] . ' ' . $form_state['complete form']['start_time']['#value']['time']);
  if ($e_date != $c_date) {
    $current_date = date("Y-m-d H:i:s");
    $start_date = strtotime($form_state['values']['start_time']);
    $end_date = strtotime($form_state['values']['end_time']);
    if ($start_date < strtotime($current_date)) {
      form_set_error('start_time', t('Date should be higher than current date.'));
    }
    if (($end_date < $current_date) || ($end_date < $start_date)) {
      form_set_error('end_time', t('End Date should not be less than the start date.'));
    }
  }
  if (!is_numeric($form_state['complete form']['conference_member_limit']['#value'])) {
    form_set_error('number of participants', t('Number of participants should be numeric.'));
  }
}

/**
 * Submit function for the form for creating the conference.
 */
function openmeetings_conference_rooms_new_conference_form_submit($form, &$form_state) {
  global $user;
  $editing = FALSE;
  if (isset($form_state['values']['conference_id'])) {
    $editing = TRUE;
  }
  // Find the moderator user ID.
  if (isset($form_state['values']['moderator_name'])) {
    preg_match('/\(uid=(\d+)\)/', $form_state['values']['moderator_name'], $matches);
    $moderator_uid = $matches[1];
  }

  // Clearing the saved emails list.
  ctools_object_cache_clear('openmeetings', 'openmeetings_added_emails');
  ctools_object_cache_clear('openmeetings', 'openmeetings_registered_participants');
  $openmeetings_organisation_id = variable_get('openmeetings_user_group_id', '');
  $notify_user = isset($form_state['values']['notify_user']) ? $form_state['values']['notify_user'] : 0;
  $disable_conference = isset($form_state['values']['disable_conference']) ? $form_state['values']['disable_conference'] : 0;
  $emails_of_non_members = array();
  if (isset($form_state['values']['added_emails']['emails']) && !empty($form_state['values']['added_emails']['emails'])) {
    $emails_of_non_members = array_keys($form_state['values']['added_emails']['emails']);
  }

  date_default_timezone_set(date_default_timezone());

  // Creating array to write conference data to local database.
  $conference_details = $form_state['values'];

  $db_time_zone = new DateTimeZone("UTC");
  // Conver strat time in UTC timezone.
  $start_time = new DateTime($form_state['values']['start_time'], $db_time_zone);
  $db_offset = $db_time_zone->getOffset($start_time);
  $db_start_time = date($form_state['values']['start_time'], time() + $db_offset);
  $utc_start_time = strtotime($db_start_time);
  // Conver end time in UTC timezone.
  $end_time = new DateTime($form_state['values']['end_time'], $db_time_zone);
  $db_offset1 = $db_time_zone->getOffset($end_time);
  $db_end_time = date($form_state['values']['end_time'], time() + $db_offset1);
  $utc_end_time = strtotime($db_end_time);

  // Max participants including moderators from our site.
  $limit = $form_state['values']['conference_member_limit'];
  if ($form_state['values']['is_moderated'] == 1) {
    $conference_member_limit = intval($limit) + 2;
  }
  else {
    $conference_member_limit = intval($limit);
  }
  $conference_details['start_time'] = $utc_start_time;
  $conference_details['end_time'] = $utc_end_time;
  $conference_details['created_by'] = $user->uid;
  $conference_details['created_on'] = REQUEST_TIME;
  $conference_details['status'] = 1;
  if ($disable_conference == 1) {
    $conference_details['status'] = 0;
  }
  $conference_details['notify_user'] = $notify_user;
  $conference_details['conference_member_limit'] = $conference_member_limit;

  // Creating conference users list.
  $conference_users = array();

  foreach ($emails_of_non_members as $user_email) {
    $conference_users[] = array(
      'uid' => '',
      'email' => $user_email,
    );
  }
  // Add moderator also to users list.
  if (isset($moderator_uid)) {
    $moderator_email_id = _get_users_as_moderator($moderator_uid);
    $conference_users[] = array(
      'uid' => $moderator_uid,
      'email' => $moderator_email_id,
    );
  }
  // If conference type is webinar,
  // change the type to restricted users before,
  // sending open meeting because,
  // this type is not available in open meeting.
  if ($form_state['values']['type'] == 2) {
    $form_state['values']['type'] = 3;
  }
  // Setting up details for creating/editing conference via SOAP API.
  $openmeetings_details = new stdClass();
  $openmeetings_details->roomname = $form_state['values']['title'];
  $openmeetings_details->type = $form_state['values']['type'];
  $openmeetings_details->max_user = $conference_member_limit;
  $openmeetings_details->is_moderated_room = $form_state['values']['is_moderated'];
  $openmeetings_details->is_audio_only = $form_state['values']['is_audio_only'];
  $openmeetings_details->allow_user_question = $form_state['values']['allow_user_questions'];
  $openmeetings = openmeetings_initialize(TRUE);
  if ($openmeetings && is_object($openmeetings)) {
    if (!$editing) {
      // Create new conference in the openmeetings.
      $created_conference_id = $openmeetings->addRoomWithModerationExternalTypeAndAudioType($openmeetings_details);

      // Add the created conference in the usergroup.
      $openmeetings->addRoomToOrg($created_conference_id, $openmeetings_organisation_id);

      // Write conference details to local database.
      $conference_details['om_cid'] = $created_conference_id;
      $success = TRUE;

      if (!drupal_write_record('openmeetings_conference', $conference_details)) {
        $success = FALSE;
      }

      $conference_id = $conference_details['cid'];
      if (_save_conference_users($conference_id, $conference_users)) {
        $success = FALSE;
      }

      // Adding moderator UID and current user as moderators.
      if (isset($moderator_uid)) {
        db_insert('openmeetings_conf_moderators')
          ->fields(array(
            'uid' => $moderator_uid,
            'cid' => $conference_id,
          ))->execute();
      }

      // Insert the current user as moderator.
      if ($user->uid != 1) {
        db_insert('openmeetings_conf_moderators')
            ->fields(array(
              'uid' => $user->uid,
              'cid' => $conference_id,
            ))->execute();
      }

      drupal_set_message(t('Successfully created new conference. Conference ID: @conf_id', array('@conf_id' => $conference_id)), 'status');
    }
    else {
      ctools_object_cache_clear('openmeetings', 'openmeetings_edited_emails');
      $conference_details['cid'] = $form_state['values']['conference_id'];
      $conference_details['om_cid'] = $form_state['values']['remote_conference_id'];
      $openmeetings_details->room_id = $form_state['values']['remote_conference_id'];
      $conference_id = $form_state['values']['conference_id'];
      $openmeetings->updateRoomWithModerationQuestionsAudioTypeAndHideOptions($openmeetings_details);
      if (!drupal_write_record('openmeetings_conference', $conference_details, array('cid'))) {
        $success = FALSE;
      }
      db_delete('openmeetings_conf_moderators')
          ->condition('cid', $conference_details['cid'])
          ->execute();
      if (_save_conference_users($conference_id, $conference_users)) {
        $success = FALSE;
      }
      // Adding moderator UID and current user as moderators.
      if (isset($moderator_uid)) {
        db_insert('openmeetings_conf_moderators')
            ->fields(array(
              'uid' => $moderator_uid,
              'cid' => $form_state['values']['conference_id'],
            ))->execute();
      }

      // Insert the current user as moderator, if it is not super admin.
      if ($user->uid != 1) {
        db_insert('openmeetings_conf_moderators')
            ->fields(array(
              'uid' => $user->uid,
              'cid' => $form_state['values']['conference_id'],
            ))->execute();
      }

      drupal_set_message(t('Saved changes of the conference, Conference ID: @conf_id', array('@conf_id' => $openmeetings_details->room_id)), 'status');
    }
    // Send emails to the attendees.
    openmeetings_send_conference_invitation_mails($conference_id);
  }
  else {
    drupal_set_message(t('There was some problem from server side, detailed information is on log.'), 'error');
  }
}

/**
 * Callback for the cancel button.
 *
 * @see openmeetings_conference_rooms_new_conference_form
 */
function openmeetings_conference_rooms_new_conference_form_cancel($form, &$form_state) {
  // Clearing the saved emails list.
  ctools_object_cache_clear('openmeetings', 'openmeetings_added_emails');
  ctools_object_cache_clear('openmeetings', 'openmeetings_registered_participants');
  drupal_goto('admin/openmeetings/conference-rooms');
}

/**
 * Function to get the email id of the user who is addded as moderator.
 *
 * @param array $moderator_uid
 *   Moderator's user id.
 *
 * @return string
 *   Email of moderator.
 */
function _get_users_as_moderator($moderator_uid) {
  $moderator_email_id = '';
  if (isset($moderator_uid)) {
    $moderator_email_id = db_query('SELECT mail
      FROM  {users}
      WHERE uid = :moderator_uid', array(':moderator_uid' => $moderator_uid))->fetchField();
  }
  return $moderator_email_id;
}

/**
 * Function to save conference users in the custom table.
 *
 * @param int $conference_id
 *   Conference ID.
 * @param array $users
 *   2 dimensional array with email ID and Drupal User ID (if present).
 *
 * @return bool
 *   TRUE if success, FALSE otherwise.
 */
function _save_conference_users($conference_id, $users) {
  if (!empty($users)) {
    $current_users = array();
    // Check if the user is already there in the database.
    $query = 'SELECT uid
      FROM {openmeetings_conf_users}
      WHERE cid = :cid
      AND uid > 0';
    $result = db_query($query, array(
      ':cid' => $conference_id,
    ));
    foreach ($result as $record) {
      $current_users[] = $record->uid;
    }
    foreach ($users as $user) {
      if (!in_array($user['uid'], $current_users)) {
        db_insert('openmeetings_conf_users')
            ->fields(array(
              'uid' => $user['uid'] != '' ? $user['uid'] : 0,
              'cid' => $conference_id,
              'email' => isset($user['email']) ? $user['email'] : '',
            ))
            ->execute();
      }
    }
    return TRUE;
  }
  return FALSE;
}

/**
 * Page callback for editing the conference.
 *
 * @param int $conference_id
 *   Confrence ID to be edited.
 *
 * @return array
 *   Renderable array of edit form.
 */
function openmeetings_conference_rooms_edit_conference($conference_id) {
  return drupal_get_form('openmeetings_conference_rooms_new_conference_form', $conference_id);
}

/**
 * Page callback to list all the conferences.
 */
function openmeetings_conference_rooms_list() {
  global $base_url;
  date_default_timezone_set(date_default_timezone());
  $filters = drupal_get_query_parameters();
  $conferences = openmeetings_get_all_conferences($filters);
  $form['filters'] = drupal_get_form('openmeetings_list_conference_filter_form', $filters);
  $header = array(
    t('ID'),
    t('Title'),
    t('Type'),
    t('Start Time'),
    t('End Time'),
    t('Created By'),
    t('Status'),
    t('Edit'),
    t('Disable'),
  );
  $rows = array();
  foreach ($conferences as $conference) {
    $conf_created_user = user_load($conference->created_by);
    $username = $conf_created_user->field_user_fullname['LANGUAGE_NONE'][0]['value'];
    $conference_types = get_conference_types();
    $rows[] = array(
      'cid' => $conference->cid,
      'title' => l($conference->title, 'openmeetings/conference/' . $conference->cid . '/view') . '<br/>' . $base_url . url('openmeetings/conference/' . $conference->cid . '/view'),
      'conference_type' => $conference_types[$conference->type],
      'start_date' => date('d-m-Y H:i:s', $conference->start_time),
      'end_time' => date('d-m-Y H:i:s', $conference->end_time),
      'created_by' => $username,
      'status' => $conference->status == 1 ? t('Enabled') : t('Disabled'),
      'edit_link' => l(t('Edit'), 'admin/openmeetings/conference-rooms/' . $conference->cid . '/edit'),
      'disable_link' => l(t('Disable'), 'admin/openmeetings/conference-rooms/' . $conference->cid . '/disable'),
    );
  }
  $form['conferences'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#sticky' => TRUE,
    '#empty' => t('No items found.'),
    '#attributes' => array(
      'class' => array('openmeetings-conference-list', 'table-collapsible'),
    ),
  );
  return $form;
}

/**
 * Function to retrieve all conference details.
 */
function openmeetings_get_all_conferences($filters = array()) {
  date_default_timezone_set(date_default_timezone());
  $query = db_select('openmeetings_conference', 'conference')
      ->fields('conference');
  if (isset($filters['show_all']) && $filters['show_all'] == 1) {
    $or = db_or();
    $or->condition('conference.status', 1);
    $or->condition('conference.status', 0);
    $query->condition($or);
  }
  else {
    $query->condition('conference.status', 1);
  }
  if (isset($filters['from_date']) && isset($filters['to_date'])) {
    $from_time_timestamp = strtotime($filters['from_date'] . "00:00:00");
    $to_time_timestamp = strtotime($filters['to_date'] . "23:59:59");
    $query->condition('conference.start_time', $from_time_timestamp, '>=');
    $query->condition('conference.start_time', $to_time_timestamp, '<=');
  }
  $result = $query->execute();
  $conferences = $result->fetchAll();
  return $conferences;
}

/**
 * Filter form for list conference page.
 */
function openmeetings_list_conference_filter_form($form, &$form_state, $filters = array()) {
  $form = array();

  $form['filter'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter'),
    '#collapsible' => FALSE,
  );
  $form['filter']['from_date'] = array(
    '#type' => 'date_popup',
    '#title' => t('Date From:'),
    '#date_format' => 'd/m/Y',
    '#date_year_range' => '-5:+5',
    '#date_label_position' => 'none',
    '#prefix' => '<div class="col">',
    '#suffix' => '</div>',
    '#default_value' => array(
      'year' => date('Y'),
      'month' => date('m'),
      'day' => date('d'),
    ),
  );
  if (isset($filters['from_date'])) {
    $form['filter']['from_date']['#default_value'] = $filters['from_date'];
  }
  $form['filter']['to_date'] = array(
    '#type' => 'date_popup',
    '#title' => t('Date To:'),
    '#date_format' => 'd/m/Y',
    '#date_year_range' => '-5:+5',
    '#date_label_position' => 'none',
    '#prefix' => '<div class="col">',
    '#suffix' => '</div>',
    '#default_value' => array(
      'year' => date('Y'),
      'month' => date('m'),
      'day' => date('d'),
    ),
  );
  if (isset($filters['to_date'])) {
    $form['filter']['to_date']['#default_value'] = $filters['to_date'];
  }
  $form['filter']['show_all'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show all conferences'),
    '#description' => t('Show both enabled and disabled conferences.'),
    '#default_value' => isset($filters['show_all']) ? $filters['show_all'] : '',
  );
  $form['filter']['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Search',
    '#submit' => array('openmeetings_list_conference_filter_form_submit'),
  );
  return $form;
}

/**
 * Submit function for list conference filter form.
 *
 * @see openmeetings_list_conference_filter_form
 */
function openmeetings_list_conference_filter_form_submit($form, &$form_state) {
  $current_path = 'admin/openmeetings/conference-rooms';
  $get_variables = array();
  if (!empty($form_state['values']['from_date'])) {
    $get_variables['from_date'] = $form_state['values']['from_date'];
  }
  if (!empty($form_state['values']['to_date'])) {
    $get_variables['to_date'] = $form_state['values']['to_date'];
  }
  if (!empty($form_state['values']['show_all'])) {
    $get_variables['show_all'] = $form_state['values']['show_all'];
  }
  if (!empty($get_variables)) {
    drupal_goto($current_path, array('query' => $get_variables));
  }
  else {
    drupal_goto($current_path);
  }
}

/**
 * Function to disable a conference.
 *
 * @param int $conference_id
 *   Conference ID.
 */
function openmeetings_conference_rooms_disable_conference($conference_id) {
  $conf_start_time = db_select('openmeetings_conference', 'oc')
          ->fields('oc', array('start_time'))
          ->condition('oc.cid', $conference_id)
          ->execute()->fetchField();
  $current_date = date("Y-m-d H:i:s");
  if ($conf_start_time > strtotime($current_date)) {
    return '<span style="color: red;"><b> Before cancelling the conference, Please notify all users about this.  </b></span>' . l(t('Edit Conference'), 'admin/openmeetings/conference-rooms/' . $conference_id . '/edit');
  }
  else {
    db_update('openmeetings_conference')
        ->fields(array(
          'status' => 0,
        ))
        ->condition('cid', $conference_id)
        ->execute();
    drupal_set_message(t('Disabled conference with conference ID: @conference_id', array('@conference_id' => $conference_id)), 'status');
    drupal_goto('admin/openmeetings/conference-rooms');
  }
}

/**
 * Function to send the invitation mails using batch API.
 *
 * @param int $conference_id
 *   The conference ID.
 */
function openmeetings_send_conference_invitation_mails($conference_id) {
  $conference = openmeetings_get_conference_details($conference_id);
  $batch = array(
    'operations' => array(
      array('openmeetings_send_conference_invitation_mail_batch', array($conference)),
    ),
    'finished' => 'openmeetings_send_conference_invitation_mail_finished',
    'title' => t('Sending E-mails to the conference attendees'),
    'init_message' => t('Sending mail process is starting.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Sending invitation mail process has encountered an error.'),
    'file' => drupal_get_path('module', 'openmeetings') . '/openmeetings.admin.inc',
  );
  batch_set($batch);
}

/**
 * Batch operation callback for sending mails.
 *
 * @param object $conference
 *   The conference object.
 * @param array &$context
 *   The context array saving details of the batch operation.
 */
function openmeetings_send_conference_invitation_mail_batch($conference, &$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_attendee_id'] = 0;

    if ($conference->notify_user == 1) {
      $results = db_select('openmeetings_conf_users', 'conf_user')
              ->fields('conf_user')
              ->condition('conf_user.cid', $conference->cid)
              ->execute()->fetchAll();
    }
    else {
      $results = db_select('openmeetings_conf_users', 'conf_user')
              ->fields('conf_user')
              ->condition('conf_user.cid', $conference->cid)
              ->condition('conf_user.email_status', 0)
              ->execute()->fetchAll();
    }
    $context['sandbox']['max'] = count($results);
  }

  $limit = 20;
  if ($conference->notify_user == 1) {
    $query = db_select('openmeetings_conf_users', 'conf')
        ->fields('conf')
        ->condition('conf.aid', $context['sandbox']['current_attendee_id'], '>')
        ->condition('conf.cid', $conference->cid)
        ->orderBy('conf.aid', 'ASC')
        ->range(0, $limit);
  }
  else {
    $query = db_select('openmeetings_conf_users', 'conf')
        ->fields('conf')
        ->condition('conf.aid', $context['sandbox']['current_attendee_id'], '>')
        ->condition('conf.cid', $conference->cid)
        ->condition('conf.email_status', 0)
        ->orderBy('conf.aid', 'ASC')
        ->range(0, $limit);
  }
  $result = $query->execute();

  while ($row = $result->fetchAssoc()) {

    $context['mail_parameters'] = variable_get('openmeetings_conference_invitation_mail_format', array());
    $context['postponed_mail_parameters'] = variable_get('openmeetings_conference_sending_postponed_meeting_details', array());
    $context['conference_object'] = $conference;

    $to = $row['email'];
    $from = $context['mail_parameters']['openmeetings_conference_email_from'];
    $context['conference_object']->uid = $row['uid'];
    if ($row['email_status'] == 1) {
      $subject = token_replace($context['postponed_mail_parameters']['openmeetings_conference_email_subject'], array('openmeetings-conference' => $context['conference_object']));
      $body = token_replace($context['postponed_mail_parameters']['openmeetings_conference_email_body'], array('openmeetings-conference' => $context['conference_object']));
    }
    else {
      $subject = token_replace($context['mail_parameters']['openmeetings_conference_email_subject'], array('openmeetings-conference' => $context['conference_object']));
      $body = token_replace($context['mail_parameters']['openmeetings_conference_email_body'], array('openmeetings-conference' => $context['conference_object']));
    }
    $headers = array(
      'Reply-to' => $from,
      'MIME-Version' => '1.0',
      'Content-Type' => 'text/plain; charset=UTF-8; format=flowed',
      'Content-Transfer-Encoding' => '8Bit',
    );
    $params = array(
      'body' => $body,
      'subject' => $subject,
      'headers' => $headers,
    );
    global $language;
    // Send the mail using drupal_mail() function.
    drupal_mail('openmeetings', 'conference-invtation-mail', $to, $language, $params, $from, TRUE);
    db_update('openmeetings_conf_users')
        ->fields(array(
          'email_status' => 1,
        ))
        ->condition('aid', $row['aid'])
        ->execute();
    $context['results'][] = check_plain($row['email']);
    $context['sandbox']['progress']++;
    $context['sandbox']['current_attendee_id'] = $row['aid'];
  }
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Finish callback for the batch process.
 */
function openmeetings_send_conference_invitation_mail_finished($success, $results, $operations) {
  if ($success) {
    // Here we do something meaningful with the results.
    $message = t('@count mails sent', array('@count' => count($results)));
  }
  else {
    // An error occurred.
    // $operations contains the remaining unprocessed.
    $error_operation = reset($operations);
    $message = t('An error occurred while processing %error_operation with arguments: @arguments', array(
      '%error_operation' => $error_operation[0],
      '@arguments' => print_r($error_operation[1],
      TRUE),
    ));
    drupal_set_message($message, 'error');
  }
}
